(****************************************************************)
(*								*)
(*	Summa.m	(Symbolic sUMs MAnipulation)			*)
(*	v2.4 English						*)
(*	for Mathematica version 2.2, 3.x 4.x 			*)
(*	(ought to work on 2.0 and 5.0 too)			*)
(*								*)
(****************************************************************)
(*
- Package 	Summa (Symbolic sUMs MAnipulation)
-			a simple package to cope with infinite and symbolic sums
-			avoiding the simplifications automatically performed by
-			Mathematica. It may prove useful when looking for
-			series solutions of ODE (Power method, Frobenius method),
-			or to do step-by-step, 'manual' symbolic manipulation of
-			symbolic finite or infinite sums.
-
-
-Author		Peltio - peltio@despammed.com
-Date		March 2005
-Version	Development
-			1.1 on Mathematica 2.2 on Win3.11.
-			1.2.x on Mathematica 3.0 on Solaris.
-			2.x on Mathematica 3.0 on Solaris and 4.x on Win9x.
-Disclaimer	This package may be freely used for non-commercial purposes,
-			as long as the above comments are kept unmodified.
-			Enhancement and correction of the package is allowed 
-			and the author looks forward to receiving comments on
-			how to improve procedure (e-mail is given above)
-Requires	FilterOptions
-
-------------------------------------------------------------------
-Version History
-
-			1.0-The packages is born as a bunch of routines 
-			designed to help the author in checking his manual 
-			solution of an ordinary differential equation.
-
-			1.1-Different Format according to Version Number and to a
-			local variable $Auto. When $Auto=True it is Sum
-			When $Auto=False it is a string looking like Sum
-
-			1.2.1-Used HoldForm to implement the display format, now
-			independent from version number
-			Corrected outRule2 in BringOut: it now treats correctly sums
-			without reference to index.
-			Modified SumCoefficient to be compatible with new outRule.
-			Added SumExpandAll.
-			Added rules for basic term by term integration.
-
-			1.2.2-Modified the rules for term by term integration
-			Added a rule to term by term differentiation
-			Added a rule to SumSimplify to automatically get rid of
-			zero sums
-			
-			2.0-Added a 'global' variable $EnableSumEvaluation that can turn on
-			and off the evalutation mechanism of Sum, by means of the
-			functions EnableSumEvaluation[] and DisableSumEvaluation[]
-
-			2.0.1-Modified the (still rough) SumCoefficient function in order 
-			to give consistent results with different versions of
-			Mathematica.
-
-			2.1-Removed the restriction that the differentiating variable
-			be a symbol. Please use with extreme caution this freedom,
-			since it can lead to nonsensical results, when misapplied.
-			
-			2.2.2-Added a second parameter to functions BringIn and
-			BringOut. It specifies a quantity that, if free from the
-			index, will be carried in or out of the Sum.
-			Modified the function SumReIndex to allow a more flexible
-			symbolic manipulation of finite sums.
-			Fixed a few help messages.
-
-			2.3-Almost every procedure has been renamed to make 
-			it start with Sum, when possible.
-			Options were added to expansion, simplification
-			differentiation and integration functions so that it
-			is now possible to use mathematica's procedures
-			at their full extent. FilterOptions needs to be
-			loaded by the package.
-			Two new procedures (SumMap and SumApply) allow 
-			given functions to be mapped and applied inside sums.
-
-			2.3.1-Added definition for the sums in the form
-			Sum[el,{k,n}]. Simple conversion to expanded form.
-
-			2.4-Added overriding and formatting defs
-			for multidimensional sums. Processing is still 
-			limited to onedimensional sums, or nested separable
-			sums. Added numerical switch for numerical endpoints.
-			Rewritten documentations of Enable- and Disable-
-			SumEvaluation but more is neeeded. 
-			It is now possible to choose whether Mma will evaluate
-			Sums with symbolic and/or numerical endpoints. 
-			Now, _this_ is useful for typesetting : ).
-			Added SumEvaluationStatus[].
-
-------------------------------------------------------------------
-Limitations
-
-			No effort has been made to make this package robust.
-			  No error message was implemented.
-			  No syntax check is performed.
-			The package was designed for onedimensional sums, 
-			although it can work with some simple (non intertwined)
-			nested sums.
-			SumD and SumIntegrate were designed to cope with simple
-			expressions, chiefly polynomial expressions inside
-			equations originating from application of power series
-			methods to o.d.e., so be careful when they are applied
-			to complicated expressions.
-			SumCoefficient must be used with utmost care since it
-			is still version dependent.
-			In short: watch out, along with many other things
-			this package enables you to aim at your own foot.
-			It's up to you not to pull the trigger.
-
-------------------------------------------------------------------
-
*)



BeginPackage["Summa`", "Utilities`FilterOptions`"]


Summa::usage=
"Abstract type to denote a symbolic or infinite sum.\n
 s=Summa[element,{k,ki,kf}]\n
 Summa objects with one or both symbolic or infinite endpoint
 are left unevaluated (and displayed as formatted Sums) to allow
 further manipulation. Summa objects with integer endpoints 
 are evaluated as built-in Sums. Summa object with symbolic or 
 infinite endpoints whose closed form is known to Mathematica 
 are kept internally unevaluated, but can be displayed either
 as unevaluated Sums or as the result of the evaluation (see
 help for SetSumFormat)."
 
EnableSumEvaluation::usage=
"EnableSumEvaluation[] enables the built-in
 mechanism of evaluation for the Sum objects in Mathematica. When this
 function is invoked it causes Mathematica to evaluate sums as it happens
 with a fresh kernel, while Summa objects will be treated and shown as
 unevaluated sums. EnableSumEvaluation[] sets the (protected) variable
 $EnableSumEvaluation to True.\n
 The option Extent can be used to estabilish what Sums have to be
 fed to Mathematica's evaluator, according to the iterator's endpoints. 
 The possible values for this option are: Symbolic, Numeric, All, None.
 Setting Extent to Symbolic (Numeric) will not affect the behavior of
 Numeric (Symbolic) sums.\n
 EnableSumEvaluation[expr] enables built-in Sum evaluation
 and substitutes every occurrence of Summa in expr with Sum, letting 
 Mathematica act on it. When the option Scope is set to Local,
 as by default, standard Sum evaluation is limited to expr; 
 with Scope->Global, standard Sum evaluation is left enabled for
 subsequent input as it happens after calling EnableSumEvaluation[]. 
 Option Verbose is used to choose whether to display or not a message
 when the effect is global. It is by default set to True."

DisableSumEvaluation::usage=
"DisableSumEvaluation[] disables the built-in
 mechanism of evaluation for the Sum objects in Mathematica, internally 
 converting every Sum into a Summa object to be manipulated by other 
 procedures (see help for Summa). After a call to DisableSumEvaluation[]
 Mathematica would leave infinite and symbolic sums unevaluated. 
 The function sets the (protected) variable $EnableSumEvaluation to False.\n
 The option Extent can be used to estabilish what Sums have to be
 protected from Mathematica's evaluator, according to the iterator's endpoints. 
 The possible values for this option are: Symbolic, Numeric, All, None.
 Setting Extent to Symbolic (Numeric) will not affect the behavior of
 Numeric (Symbolic) sums.\n
 DisableSumEvaluation[expr] disables the built-in Sum evaluation
 and then substitutes every occurrence of Sum in expr with Summa.
 When option Scope is set to Local, as by default, this effect is
 limited to expr; with Scope->Global, builtin Sum evaluation is left off
 for subsequent input, as after DisableSumEvaluation[].  
 Option Verbose is used to choose whether to display or not a message 
 when the effect is global. It is by default set to True."

Scope::usage=
"Scope is an option for EnableSumEvaluation and DisableSumEvaluation.
 It is used to estabilish whether the built-in evaluation of Sum should
 be changed only for the expression passed to the function (Scope->Local),
 or has to affect subsequent instructions (Scope->GLobal). In the 
 latter case, if option Verbose is set to True (as is by 
 default) a message will be issued to warn the user about the change 
 occurred to the global environment."
 
Extent::usage=
"Extent ins an option for EnableSumEvaluation and DisableSumEvaluation.
It is used to choose what kind of sums should (or should not) be passed
to Mathematica's evaluator. Legal values are Symbolic, Numeric, All, None."

SumEvaluationStatus::usage=
"SumEvaluationStatus[] shows the values of the environmental variables
$EnableSumEvaluation (concerning the evaluation of Sums with symbolic
or infinite endpoints) and $EnableNumericSum (concerning the evaluation
of Sums with numerical - integer - endpoints)."

SetSumFormat::usage=
"SetSumFormat[Automatic] allows automatic display of
 Summa objects as if they were a built-in Sum. This setting affects
 the display format only. To have the expression behave as displayed
 it is necessary to replace Summa with Sum by means of SummaToSum[expr]./n
 SetSumFormat[Inhibithed] displays Summa objects like built-in Sums but
 leaves them totally unevaluated, avoiding automatic kernel 
 simplifications."
 
SummaToSum::usage=
"SummaToSum[expr] converts every occurrence of Summa in expr into Sum.
 When Sum evaluation is enabled this result in the automatic evaluation
 of the sums and the replacement of every convergent series with its closed
 form sum. When Sum evaluation is disabled, this operatin is pretty useless."
 
SumToSumma::usage=
"SumToSumma[expr] converts every occurrence of Sum in expr into Summa.
 The attribute HoldAll prevents the evaluation of sums passed to it, 
 so that it can be used when ordinary sum evaluation is enabled. It can be used
 to obtain a version of expr that won't evaluate and that allows further
 manipulaton by the routine of the package."

Automatic::usage="Argument of SetSumFormat, send the output form of Summa
 to kernel before displaying"
Inhibited::usage="Argument of SetSumFormat, does not send the output form
 of Summa to kernel before displaying"

BringOut::usage=
"BringOut[expr] brings out of every sum in expr
 everything that does not contains the index of the sum.\n
 BringOut[expr, c] divides the addendum of the sum by c 
 and multiplies the whole sum by the same amount. The quantity
 c must not depend on the index."

BringIn::usage=
"BringIn[expr] brings the eventual factor pre-multiplying a
 sum inside the sum symbol.In doing so, if the internal 
 variable $Explode is set to True, it performs an automatic
 expansion of what is inside.\n
 BringIn[expr, c] multiplies the addendum of the sum by c and 
 divides the whole sum by the same amount. The quantity
 c must not depend on the index."

SumExpand::usage=
"SumExpand[expr] expands what is inside every sum and
 makes a separate sum for each term. It is usually used in connection with
 BringIn. This function shares the same options as Expand."

SumExpandAll::usage= 
"SumExpandAll[expr] repeatedly expands what is inside
 every sum and makes a separate sum for each term until the expression no
 longer changes. It also brings premultiplying factors inside the sums.
 This function shares the same options as ExpandAll."

SumSimplify::usage=
"SumSimplify[expr] reduce all the sums with the same
 indexes (that is with the same index name, start and final values) to
 one single sum. This function shares the same options as Simplify."
 
SumFullSimplify::usage=
"SumFullSimplify[expr] is analogous to SumSimplify, but uses FullSimplify
in order to try a more extensive set of transofrmations. 
This function shares the same options as FullSimplify."

SumSimplifyEach::usage=
"SumSimplifyEach[expr] simply maps Simplify
 onto the elements of each sum in expr. It does not affect whatever is
 outside. This function shares the same options as Simplify"
 
SumFullSimplifyEach::usage=
"SumFullSimplifyEach[expr] is analogous to SumSimplifyEach, but uses FullSimplify
in order to try a more extensive set of transformations. 
This function shares the same options as FullSimplify."

SumSplit::usage=
"SumSplit[expr,n0] breaks in two all the sums contained
 in expr so that one of them will start from the value n0.
 The sums with numerical starting and ending points are automatically
 expanded (as is for Sum)."

SumReIndex::usage=
"SumReIndex[expr,k->L[k]] shifts the index as
 indicated in L[k]. The shift affects every sum contained in expr that has
 index k. It is not allowed to change the index identificator.\n
 Example: SumReIndex[expr,k->k-2]"

SumIndexAlign::usage=
"SumIndexAlign[expr,{k,k0}] shifts the index of every sum
 that is in expr and has index k, in order to have them all to start from
 value k0."

SumExponentAlign::usage=
  "SumExponentAlign[expr,x,k->L[k]] shifts the index of every sum
 that is in expr, has index k and contains x^(f[k]) in order to
 have all of them with the same power of x: x^L[k]. It is not
 allowed to change the index identifier. x may be (x-x0) or whatever.\n
 This procedure automatically expands the elements of each sum before
 searching for the exponent of x. The default expansion can be avoided by
 setting the option Explode->False. This is advisable when coping with
 powers of (x-x0)."

Explode::usage=
"Explode is an option of SumExponentAlign.\n
 When set to False it avoids the automatic expansion of the elements
 of the sums performed before the index shift. This is advisable when
 coping with powers of (x-x0)."

SumD::usage=
"SumD[expr,x] and SumD[expr,{x,n}] differentiate with respect
 to x, n times. This procedure can be applied directly to equations.
 It has an option Inside that when set to False avoids to automatically
 bring inside the factors premultiplying the sums. 
 This function shares the same options as D."

Inside::usage=
"Inside is an option of SumD and SumIntegrate.\n
 When set to False avoids to automatically bring inside the factors
 premultiplying the sums."

SumDEach::usage=
"SumDEach[s,x] and SumDEach[s,{x,n}] differentiate with respect
 to x, n times. This procedure can be applied only to sums (and sums of sums)
 since it operates exclusively by moving D inside the sum  symbol. When
 used with mixed expressions it gives wrong results:\n
 SumDEach[f[x] + Summa[g[x],{k,n1,n2}]=f[x] + Summa[g'[x],{k,n1,n2}].
 This function shares the same options as D."

SumIntegrate::usage=
"SumIntegrate[expr,x] and SumIntegrate[expr,{x,a,b}]
 integrate term by term with respect to x. When the option Inside is set to
 False integrals not depending on the index variable are brought outside
 the summa objects.
 This function shares the same options as Integrate."

SumToTerm::usage=
"SumToTerm[expr] replace the sums with their terms, by simply dropping
the Sum structure and the iterator."

SumCoefficient::usage=
"SumCoefficient[Summa[el,it],x] returns
 Coefficient[el,x].\nSumCoefficient[expr,x,0] returns expr when all
 the powers of x are set to zero."

GetExponent::usage=
"GetExponent[poly,x] gives the exponent of x inside poly.
It works with non numerical exponents too."

SumPosition::usage=
"SumPosition[expr,pos] is still and undocumented feature"

SumMap::usage=
"SumMap[f,expr]
 SumMap[f,expr,lev] is still and undocumented feature"

SumMapAt::usage=
"SumMapAt[f,expr,pos]
 SumMapAt[f,expr,pos,lev] is still and undocumented feature"

SumApply::usage=
"SumApply[f,expr] is still and undocumented feature"

SumApplyAt::usage=
"SumApply[f,expr,pos] is still and undocumented feature"


(* Aliases for procedures names *)

GetElement=SumToTerm;
GetCoefficient=SumCoefficient;
(*GetExponent;*)
ExpandSum=SumExpand;
ExpandAllSum=SumExpandAll;
SimplifySum=SumSimplify;
SimplifyEachSum=SumSimplifyEach;
FullSimplifySum=SumFullSimplify;
FullSimplifyEachSum=SumFullSimplifyEach;
(*BringIn;*)
(*BringOut;*)
BreakSum=SumSplit;
ReIndexSum=SumReIndex;
AlignIndex=SumIndexAlign;
AlignExponent=SumExponentAlign;
DSum=SumD;
DEachSum=SumDEach;
IntegrateSum=SumIntegrate;


(* Declares and protects the 'global' variables used to switch
on and off the builtin evaluation mechanism of mathematica *)

		Unprotect[$EnableSumEvaluation];
		$EnableSumEvaluation=False;
		Protect[$EnableSumEvaluation];
		(* used by Sum, when False, sums are converted
		to Summa object and won't be evaluated by
		mathematica unless explicitly required *)
		
		Unprotect[$EnableNumericSum];
		$EnableNumericSum=True;
		Protect[$EnableNumericSum];
		(* used by Summa, when True, sums with numerical 
		endpoints are evaluated directly by mathematica *)
		
(*makes them global symbols if they are not in a particular version*)
Local;Global;All;None;Symbolic;Numeric;Verbose;



Begin["`Private`"]

(* ==============================================================*)
(* Definitions --------------------------------------------------*)
(* ==============================================================*)

(* Internal Variables *)

$Semplifica=True;		(* used by SumExponentAlign *)
$Explode=True;			(* used by SumSimplify *)
$Auto=False;			(* used by Format *)


(*  SUM overriding ----------------------------------------------
	Main definition that overrides the builtin Sum when 
	$EnableSumEvaluation=False
	but allows the builtin Sum evaluation inside the 
	Block[] structure with the local value True for the 
	protected variable. The first definition has problems
	with nested sums: only the outer one would be 
	replaced by Summa - needs fixing for n-dim sums
*)

(*
		Unprotect[Sum];
		Sum[args__]/; $EnableSumEvaluation == False:=
			Block[
				{$EnableSumEvaluation=True},
				Summa[args]
			]
		Protect[Sum];
*)

(* nested sum hack: will be added under a test condition *)

		Unprotect[Sum];
		Sum[args__] /; $EnableSumEvaluation == False := 
		  Block[{$EnableSumEvaluation = True},
		    Summa[
		      Sequence @@ Block[
		          {$EnableSumEvaluation = False},
		          tmp[args]]]
		    ]
		Protect[Sum];
(* *)

(* 
	Procedures to enable and disable the built-in
	Sum evaluation process for Sum objects, either globally or
	locally, with replacement in the expression passed to them.
	By default, Numerical sum evaluation is enabled and 
	Symbolical sum evaluation is disabled. Called without options
	EnableSumEvaluation enables built-in eval for numerical and
	symbolic sums, while DisableSumEvaluation only disables
	numerical sum evaluation.
	One option and two procedures allow the users to take
	care of every possibile case.
*)

(* Internal procedure to set protected variables *)

Attributes[setProtectedVar] = {HoldAll};

setProtectedVar[var_, value_] := (
	Unprotect[var];
    var = value;
    Protect[var];
    )

(* Enable and Disable sum evaluation for
		Numeric only
		Symbolic (and infinite) only
		Numeric and Symbolic (All)
	sums, according to the value of the Extent option.
	Enabling with Extent->All is equivalent to disabling with
	Extent->None and viceversa. Extent->Numeric will not affect
	the status for Symbolic evaluation and viceversa.
	Still in beta.
*)
		
    
Options[EnableSumEvaluation] = {
      Scope -> Local,
      Extent -> All,
      Verbose -> True
      };
Options[DisableSumEvaluation] = {
      Scope -> Local,
      Extent -> Symbolic,
      Verbose -> True
      };

Attributes[EnableSumEvaluation] = {HoldFirst};
Attributes[DisableSumEvaluation] = {HoldFirst};

EnableSumEvaluation[opt___Rule] := Module[
    {ext, extStr},
    ext = (Extent /. {opt} /. Options[EnableSumEvaluation]);
    Switch[ ext,
      Symbolic, 
      	setProtectedVar[$EnableSumEvaluation, True];
      	extStr="From now on Sums will be passed to Mathematica for evaluation.",
      Numeric, 
      	setProtectedVar[$EnableNumericSum, True];
      	extStr="From now on Sums and Summa objects with numerical endpoints will be passed to Mathematica for evaluation.",
      All,
        setProtectedVar[$EnableSumEvaluation, True]; 
        setProtectedVar[$EnableNumericSum, True];
      	extStr="From now on not only all Sums, but also Summa objects with numerical endpoints will be passed to Mathematica for evaluation.",
      None,
        setProtectedVar[$EnableSumEvaluation, False]; 
        setProtectedVar[$EnableNumericSum, False];
      	extStr="From now all Sums, and Summa objects with numerical endpoints, will not be evaluated."
      ];
    If[(Verbose /. {opt} /. Options[EnableSumEvaluation]), 
      Print[extStr]
      ];
    ]

EnableSumEvaluation[expr_, opt___Rule] := 
Module[{tmpexpr,oldNum,oldSym},
    oldNum=$EnableNumericSum;
    oldSym=$EnableSumEvaluation;
    
    EnableSumEvaluation[opt, Verbose -> False];
    tmpexpr = expr /. Summa -> Sum;    
    If[(Scope /. {opt} /. Options[EnableSumEvaluation]) === Local,
    
      setProtectedVar[$EnableSumEvaluation, oldSym];
      setProtectedVar[$EnableNumericSum, oldNum];
      tmpexpr /. Sum -> Summa,
      
      tmpexpr
    ]
]


DisableSumEvaluation[opt___Rule] := Module[
    {ext, extStr},
    ext = Extent /. {opt} /. Options[DisableSumEvaluation];
    Switch[ ext,
      Symbolic, 
      	setProtectedVar[$EnableSumEvaluation, False];
      	extStr="From now on Sums with Symbolic endpoints will be turned 
 into Summa objects that will not be evaluated by Mathematica.
 Summa objects are formatted and shown as ordinary Sums (provided
 there are no explicit instructions to evaluate them).",
      Numeric, 
      	setProtectedVar[$EnableNumericSum, False];
      	extStr="From now on Summa objects with numerical endpoints
 will not be evaluated by Mathematica. This will affects Sums
 only if their evaluation has already been inhibited.",
      All, 
        setProtectedVar[$EnableSumEvaluation, False]; 
        setProtectedVar[$EnableNumericSum, False];
       	extStr="From now on all Sums will be turned into Summa objects
 that will not be evaluated by Mathematica. Summa objects are formatted
 and shown as ordinary Sums.",
      None,
        setProtectedVar[$EnableSumEvaluation, True]; 
        setProtectedVar[$EnableNumericSum, True];
      	extStr="From now on not only all Sums, but also Summa objects
 with numerical endpoints will be passed to Mathematica for evaluation."
      ];
    If[(Verbose /. {opt} /. Options[DisableSumEvaluation]), 
      Print[extStr]
      ];
    ]


DisableSumEvaluation[expr_, opt___Rule] := 
Module[{tmpexpr,oldNum,oldSym},
    oldNum=$EnableNumericSum;
    oldSym=$EnableSumEvaluation;
    
    DisableSumEvaluation[opt, Verbose -> False];
    tmpexpr = expr /. Sum -> Summa;    
    If[(Scope /. {opt} /. Options[DisableSumEvaluation]) === Local,
    
      setProtectedVar[$EnableSumEvaluation, oldSym];
      setProtectedVar[$EnableNumericSum, oldNum];
      tmpexpr /. Summa -> Sum,
      
      tmpexpr
    ]
]

SumEvaluationStatus[] := StringJoin[
    "$EnableSumEvaluation = ",
    ToString[$EnableSumEvaluation],
    "\n",
    "$EnableNumericSum = ",
    ToString[$EnableNumericSum]
    ]

Attributes[SumToSumma]={HoldAll};
SummaToSum[expr_]:=expr//.Summa->Sum
SumToSumma[expr_]:=ReleaseHold[Hold[expr]//.Sum->Summa]


(* SUMMA --------------------------------------------------------
	This is the definition of the function Summa that will
	be automatically called by Sum as long as the variable 
	$EnableSumEvaluation is set to False
	For symbolic and infinite sums it must return unevaluated.
	For numerical endpoints it returns the standard evaluated Sum
	unless $EnableNumericSum is set to False.
*)
	

	(* unevaluated for symbolic -and infinite- endpoints *)

		Clear[Summa];

	(* built-in, evaluated, Sum when both endpoints are integers *)
	(* needs to be extended to numeric finite values *)
	(* this definition covers two types of indexing: *)
	(* {k,n0,n1} and {k,n}  *)	
	
	(* Converting iterators to expanded form *)
	
		Summa[el_, it:(({_, _} | {_, _, _})..)] /; 
			EvenQ[Times @@ Map[Length, {it}, -1]] :=
			Summa[el,ReleaseHold[Hold[it] /. {k_,k1_} :> {k,1,k1}]]
	
	(* Automatic evaluation of sums with integer endpoints *)
	(* To disable this behaviour set $EnableNumericSum to False via *)
	(* the EnableSumEvaluation[] and DisableSumEvaluation[]*)
	(* procedures *)
	
		Summa[el_, 
			it:({_Symbol,_Integer,_Integer}|{_Symbol,_Integer})..] /; $EnableNumericSum==True :=
			Sum[el, it]
	
	(* previous onedimensional only version without the numeric switch*)
	(*
		Summa[el_,{k_Symbol,k0_Integer,k1_Integer}]:=
			Block[
				{$EnableSumEvaluation=True},
				Sum[el,{k,k0,k1}]
			]
	*)			

(* Added to format Summa and to allow for the simplification
of convergent infinite sums (such as Sum[1/k^2,{k,0,Infinity]
when $Auto=True (or equivalently SetSumFormat[Automatic]is invoked.
The Block allows for the built-in definition of Sum to be applied
without being reduced to Summa, thus avoiding infinite recursion  *)

SetSumFormat[Automatic]:=($Auto=True;)
SetSumFormat[Inhibited]:=($Auto=False;)

Format[Summa[el_, it:(({_, _} | {_, _, _})..)]] :=
  Block[
    {$EnableSumEvaluation=True},
    If[$Auto,
      	Sum[el, it],
      	HoldForm[Sum[el, it]]
    ]
  ]



(*
   ==============================================================
	Manipulation procedures
   ==============================================================

>From now on the package defines the rules and the simplification 
functions for the infinite and symbolic sums. Attaching these
definitions to Summa will make the rules apply to Sum only if 
$EnableSumEvaluation is set to False (a condition that turns every 
Sum in the input into a Summa to which  the rules apply.

	When $EnableSumEvaluation = False...
	====================================
	All functions applied to Summa(s) are applied to Sum(s) too.
	So it is possibile to use either Sum or Summa to represent 
	infinite and symbolic sums to be manipulated.
	
	When $EnableSumEvaluation = True...
	===================================
	The built-in evaluation process of Sums is restored.
	It is then not advisable to use the simplification routines 
	on Sum(s).They might give unevaluated or even erroneous results.
	It is still possible, though, to apply them to Summa(s), thus 
	achieving the same functionality of the previous version of the 
	package.
*)






(* BringOut and SumExpand+SumExpandAll--------------------------*)


outrule1={
		Summa[f_+g_,it:{k_,__}]:> Summa[f,it]+Summa[g,it],
		Summa[c_ f_,it:{k_,__}]:> c Summa[f,it]/;FreeQ[c,k]
		};
outrule2=Summa[c_,it:{k_,__}]:> c Summa[1,it]/;FreeQ[c,k];
outrules=Flatten[{outrule1,outrule2}];
outrule3[x_] = Summa[el_, it:{k_,__}] :> x*Summa[el/x, it] /; 
     FreeQ[x, k];

BringOut[s_]:=s//.outrules
BringOut[s_, x_] := s /. outrule3[x]


SumExpand[s_,opts___Rule]:=ExpandAll[BringOut[s],opts]
SumExpandAll[s_,opts___Rule]:=FixedPoint[ExpandAll[BringOut[#],opts]&,s]//BringIn

Options[SumExpand]=Options[ExpandAll]
Options[SumExpandAll]=Options[ExpandAll]






(* BringIn, SumSimplify e SumFullSimplify -------------------------------------*)


inrule1[opts___Rule]:= c_ Summa[el1_,it:{k_,__}]:>
		Summa[If[$Explode,
				Expand[c el1,FilterOptions[Expand,{opts}]],
				c el1]	,it] /; FreeQ[c,k];
inrule2[opts___Rule]:=Summa[el1_,it_]+Summa[el2_,it_]:>
		Summa[If[$Semplifica,
				Simplify[el1+el2,FilterOptions[Simplify,{opts}]],
				el1+el2],it];
inrule2f[opts___Rule]:=Summa[el1_,it_]+Summa[el2_,it_]:>
		Summa[If[$Semplifica,
				FullSimplify[el1+el2,FilterOptions[FullSimplify,{opts}]],
				el1+el2],it];				
inrule3[x_] = Summa[el_, it:{k_,__}] :> (1/x)*Summa[x*el, it] /; 
     FreeQ[x, k];
     

BringIn[s_,opts___Rule]:=s /.inrule1[opts]
BringIn[s_, x_] := s /. inrule3[x]

SumSimplify[s_,opts___Rule]:=
	s /.inrule1[opts] //.inrule2[opts] /.Summa[0,_]->0
SumSimplifyEach[s_,opts___Rule]:=
	s /. Summa[el_,it_]:>Summa[Simplify[el,opts],it]/.Summa[0,_]->0


SumFullSimplify[s_,opts___Rule]:=
	s /.inrule1[opts] //.inrule2f[opts] /.Summa[0,_]->0
SumFullSimplifyEach[s_,opts___Rule]:=
	s /. Summa[el_,it_]:>Summa[FullSimplify[el,opts],it]/.Summa[0,_]->0


Options[BringIn]=Options[Expand]
Options[SumSimplify] = Union[Options[Simplify], 
	DeleteCases[Options[Expand],Trig->_]]
Options[SumFullSimplify] = Union[Options[FullSimplify], 
	DeleteCases[Options[Expand],Trig->_]]
Options[SumSimplifyEach]=Options[Simplify]
Options[SumFullSimplifyEach]=Options[FullSimplify]







(*SumSplit -----------------------------------------------------*)

ruleSpezza[n_]=Summa[el_,{k_,ko_,k1_}]:>
	Summa[el,{k,ko,n-1}]+Summa[el,{k,n,k1}];

SumSplit[s_,n_Integer]:=s /. ruleSpezza[n]







(*SumReIndex, SumIndexAlign, SumExponentAlign -------------------*)
(* the form Summa[el,{k,k1}] is covered by the transformation
 to Summa[el,{k,1,k1}] previously defined *)


(* old version, leave it commented
	ruleIndex[k_->j_]=Summa[el_,{k,ko_,k1_}]:>
	Summa[el/.k->j,{k,ko-j+k,k1-j+k}]; 
*)
	
ruleIndex[k_, newk_]=Summa[el_, {k, ko_, k1_}] :> 
	Summa[el/.k->newk, 
     {k,k/.First[Solve[newk==ko,k]], k/.First[Solve[newk==k1,k]]}
     ] /; !FreeQ[newk, k]	
	
SumReIndex[s_,k_->newk_]:=s /.ruleIndex[k,newk]


indexShift[{k_,o_}]=Summa[el_,{k,ko_,k1_}]:>
	Summa[el/.k->k-o+ko,{k,o,k1-ko+o}];
indexDownShift[{k_,o_}]=Summa[el_,{k,ko_,k1_}]:>
	Summa[el/.k->k-o+k1,{k,ko-k1+o,o}];
	
SumIndexAlign[s_,{k_,o_}]:=s /.indexShift[{k,o}]
SumIndexAlign[s_,{k_,o_},Direction->Down]:=s/.indexDownShift[{k,o}]


GetExponent[expr_,x_]:=Flatten[Cases[{expr},x^n_ ->n,Infinity]][[1]]

powerShift[x_,k_->kin_]=Summa[el_,{k,k0_,k1_}]:>
	Summa[el/.k->kin-GetExponent[el,x]+k,{k,k0-kin+GetExponent[el,x],
        k1-kin+GetExponent[el,x]}];
        
Options[SumExponentAlign]={Explode->True,Inside->True};
									(*Future Expansion*)
									
SumExponentAlign[s_,x_,k_->kin_,opts___Rule]:=
	Module[
		{opt1,res},
		opt1=Explode/.{opts}/.Options[SumExponentAlign];
		res=If[opt1,
				BringIn[SumExpand[s]],
				$Explode=False;BringIn[s]
				]/.powerShift[x,k->kin];
		$Explode=True;
		res
	]








(*SumD --------------------------------------------------------*)
(* Limitations:
-	SumD was designed to operate on equations with polinomial
-	expressions. It works on more complex expressions, though.
-	Version (v1.0) automatically brought in 
-	factors premultiplying the sums. Later versions require 
-	to specify this operation whenever needed.
-	
-	Removed Symbol matching of differentiating variable x
-	Added the same options used by D (Nonconstants)
-	Reordered function definition from more specific
-	to more general.
*)
Clear[f,f1,f2,c,d,x,dummy]

diffRule0[x_]=
	{
	d[c_ f_]:>c d[f] /; FreeQ[c,x],
	d[f1_+f2_]:>d[f1]+d[f2]
	};
diffRule1[x_,opts___Rule]:=
	{
	d[f1_ f2_]:>d[f1]f2+f1 d[f2],
	d[f_^c_]:>c f^(c-1) d[f] /; FreeQ[c,x],
	d[f1_/f2_]:>(d[f1] f2-f 1d[f2])/f2^2,
	d[f1_[f2_]]:> d[f2](D[f1[dummy],dummy,opts]/.dummy->f2)
	};
diffRule[x_,opts___Rule]:= Flatten[{diffRule0[x],diffRule1[x,opts]}];
diffRule2[x_,opts___Rule]:=
	{
	d[Summa[el_,it_]]:>Summa[D[el,x,opts],it],
	d[f_]:>D[f,x,opts]/; FreeQ[f,Summa]
	};
	
(* After adding opts_Rule, the following two lines had to be moved up to gain
due precedence over the less specialized SumD[s_,x_,opts___] *)

SumD[lhs_==rhs_,x_,opts___Rule]:=SumD[lhs,x,opts]==SumD[rhs,x,opts]
SumD[s_,{x_,n_},opts___Rule]:=Nest[SumD[#,x,opts]&,s,n]
SumD[s_,x_,opts___Rule]:=
	Module[
		{opt1,opt2},
		opt1=Inside/.{opts}/.Options[SumD];
		opt2=FilterOptions[D,{opts}];
		d[If[opt1,BringIn[s],s]]//.diffRule[x,opt2]/.diffRule2[x,opt2]
	]

SumDEach[s_,{x_,n_},opts___Rule]:=s/.Summa[el_,it_]:>Summa[D[el,{x,n},opts],it]
SumDEach[s_,x_,opts___Rule]:=s/.Summa[el_,it_]:>Summa[D[el,x,opts],it]


Options[SumD]=Union[{Inside->True},Options[D]];
(*Added options for D *)
Options[SumDEach]=Options[SumD]








(* SumIntegrate ---------------------------------------------- *)
(*	Added the same options as integrate *)


intRule1[x_Symbol|{x_Symbol,_,_}]=
	{
	in[f1_+f2_]:>in[f1]+in[f2],
	in[c_ f_]:>c in[f] /; FreeQ[c,x],
	in[f1_ f2_]:>in[BringIn[Expand[f1 f2]]]
	};
intRule2[x_,opts___Rule]:=
	{
	in[Summa[el_,it_]]:>Summa[Integrate[el,x,opts],it],
	in[f_]:>Integrate[f,x,opts] (* /; FreeQ[f,Summa] *)
	};
	
(* After adding opts_Rule, the following line had to be moved up to gain
due precedence over the less specialized SumIntegrate[s_,x_,opts___] *)

SumIntegrate[lhs_==rhs_,x_,opts___Rule]:=
  SumIntegrate[lhs,x,opts]==SumIntegrate[rhs,x,opts]

SumIntegrate[s_,x_,opts___Rule]:=
	Module[
		{opt1,opt2},
		opt1=Inside/.{opts}/.Options[SumIntegrate];
		opt2=FilterOptions[Integrate,{opts}];
		If[opt1,
			in[BringIn[s]] //.intRule1[x]/.intRule2[x,opt2],
			BringOut[in[s]//.intRule1[x]/.intRule2[x,opt2]]
		]
	]
	
Options[SumIntegrate]=Union[{Inside->True},Options[Integrate]];
(*Added options for Integrate*)







(*Element extraction: SumToTerm and SumCoefficient ------------*)
(* NOTES:
-	SumCoefficient must be used with the utmost care 
-	it is version dependent, since it relies on the
-	version dependent procedure Coefficient
*)


getElement = Summa[el_,it_] :> el;
SumToTerm[s_] := s /. getElement

(*code for all versions*)
SumCoefficient[Summa[el_,it_],x_,k___]:=SumCoefficient[el,x,k]
SumCoefficient[s_ /;Length[Cases[s,_Summa,Infinity]]==1 ,x_,k___]:=
  SumCoefficient[Cases[s,_Summa,Infinity][[1]],x,k]

SumCoefficient[el_,x_]:=Coefficient[el/.Summa[0,_]->0,x];
SumCoefficient[el_,x_,0]:=el/.{x^_->0,x->0}/.Summa[0,_]->0;
SumCoefficient[el_,x_^k_,n_:1]:=SumCoefficient[el,x,k* n];

If[$VersionNumber>=3,
  
  (*code for later versions*)
  SumCoefficient[el_,x_,n_]:=Coefficient[el/.Summa[0,_]->0,x,n],
  
  (*code for earlier versions*)
  SumCoefficient[el_,x_,n_]:=
    Block[
      {res=Coefficient[el/.Summa[0,_]->0,x^n]},
      If[FreeQ[res,x],res,0]
    ]
]









(* Mapping and applying functions to Summa terms ------------*)

SumPosition[expr_, k_] := Part[Position[expr, Summa[_, _]], k]

SumMap[f_, expr_] := expr /. Summa[el_, it_] :> Summa[f[el], it]
SumMap[f_, expr_, 0] := SumMap[f, expr]
SumMap[f_, expr_, n_] := expr /. Summa[el_, it_] :> 
    Summa[Map[f, el, n], it]
    
SumMapAt[f_, expr_, pos_] := MapAt[f, expr, SumPosition[expr, pos]]
SumMapAt[f_, expr_, pos_, lev_] := 
  MapAt[Map[f, expr, lev], expr, SumPosition[expr, pos]]
    

SumApply[f_, expr_, n_:1] := expr /. Summa[el_, it_] :> 
    Summa[Apply[f, el, n], it]


(****************************************************************)


End[]

Protect[
	Summa, SetSumFormat,
	EnableSumEvaluation, DisableSumEvaluation, SumEvaluationStatus,
	SumToSumma, SummaToSum,
	SumMap, SumApply,
	
	SumToTerm, SumCoefficient,
	SumExpand, SumExpandAll, SumSimplify, SumSimplifySum,
	SumFullSimplify, SumFullSimplifyEach,
	SumSplit, SumReIndex, SumIndexAlign, SumExponentAlign,
	SumD, SumDEach, SumIntegrate,
	SumPosition, SumMap, SumMapAt, SumApply, SumApplyAt,
	
	GetExponent, BringOut, BringIn, 
	
	GetElement, GetCoefficient, GetExponent,
	ExpandSum, ExpandAllSum, SimplifySum,
	FullSimplifySum, FullSimplifyEachSum,
	BringIn, BringOut,
	BreakSum, ReindexSum, AlignIndex,
	AlignExponent,
	DSum, DEachSum, IntegrateSum
	
	(* , Scope, Verbose, Explode, Inside, Symbolic, Numeric, Extent *)
]

EndPackage[]

(* (c)2005 Peltio Inc. *)
