Notebook[{
Cell["Basic Overriding", "Title"],

Cell[TextData[{
  "Enhancing built-in procedures and objects avoiding ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s evaluator"
}], "Subtitle"],

Cell["(c) 2005  Peltio (peltio@despammed.com)", "Copyright"],

Cell["\<\
The main change introduced with version 2 of the package Summa.m is \
the ability to address Summa objects with exactly the same input syntax used \
for built-in Sums. In the rest of this notebook we will address, somewhat \
loosely, this behavior by \"enabling and disabling the Sum evaluation process\
\". The built-in evaluation process is not actually being inhibithed: it just \
doesn't have a chance to see Sums anymore unless the user sets a specific \
environment variable that suppresses the conversions of Sum into Summa.\
\>", 
  "Text"],

Cell[TextData[{
  "It is possibile to redefine a built-in procedure in such a way that it \
will call itself without incurring in an infinite recursion. I am giving here \
just a brief outline of the methods involved, postponing a thorough \
discussion to a separate notebook (\"Overriding built-in objects\", whose \
associated package 'Ovverride.m' is still under development). The key idea is \
to use an 'environmental variable', say ",
  StyleBox["$enablebuiltin",
    FontWeight->"Bold"],
  ", that switches the built-in evaluation process off in order to give way \
to user defined code. \nWe initialize (and protect) the variable to False in \
order to prevent ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " from taking over our code."
}], "Text"],

Cell[BoxData[
    \($enablebuiltin = False; \nProtect[$enablebuiltin]; \)], "Input",
  CellFrame->True,
  Background->GrayLevel[0.833326]],

Cell[CellGroupData[{

Cell["Enhancing built-in procedures", "Section"],

Cell[TextData[{
  "If we wish to redefine a built-in procedure, let's call it ",
  StyleBox["BuiltinProcedure",
    FontWeight->"Bold"],
  ",  in term of itself we just have to define it with a conditioned argument \
that applies only when ",
  StyleBox["$enablebuiltin = False",
    FontWeight->"Bold"],
  " and enclose the code that call the original procedure in a ",
  StyleBox["Block",
    FontWeight->"Bold"],
  " that sets ",
  StyleBox["$enablebuiltin = True",
    FontWeight->"Bold"],
  ". In this way a call to BuiltinProcedure will be passed to the original \
built-in procedure. Unprotecting and Protecting is necessary in order to \
allow to modifiy the code associated with the built-in procedure. The \
following replaces the built-in procedure with a simple call to a user \
defined prodcedure called ",
  StyleBox["userCode",
    FontWeight->"Bold"]
}], "Text"],

Cell[BoxData[
    \(Unprotect[BuiltinProcedure]; \n
    BuiltinProcedure[args___, opts___Rule\ ] /; $enablebuiltin \[Equal] 
          False := 
      Block[{$enablebuiltin = True}, 
        \[IndentingNewLine]userCode[args, opts]\[IndentingNewLine]]; \n
    Protect[BuiltinProcedure]; \)], "Input",
  CellFrame->True,
  Background->GrayLevel[0.833326]],

Cell[TextData[{
  "Once we've done this, we can write whichever code is needer for ",
  StyleBox["userCode ",
    FontWeight->"Bold"],
  "(or we could not. In fact, we migth also want to replace the original \
object with a wrapper that does just nothing - this is indeed the case for \
Summa). Since our code is called from withing a Block that sets the \
environmental variable $enablebuiltin to True, every call to BuiltinProcedure \
from within it will invoke the procedure originally used by ",
  StyleBox["Mathematica.",
    FontSlant->"Italic"],
  " The following sample code can be used to show that.is is so:"
}], "Text"],

Cell[BoxData[
    \(userCode[args___, opts___Rule] := 
      \[IndentingNewLine]\((
        \[IndentingNewLine]Print[
          "\<The variable $enablebuiltin is temporarily \>", $enablebuiltin, 
          \ "\<. \n The following function calls BuiltinProcedure without any \
condition on the arguments: it is the built-in one.\>"\ ]; 
        \[IndentingNewLine]myFun[BuiltinProcedure[args, opts]]
          \[IndentingNewLine])\)\)], "Input"],

Cell["\<\
Now, when we call the built-in procedure in the global environment \
(where $enablebuiltin is set to False) we are in fact calling our code that, \
in turn, can call the original built-in function after having properly set \
the value of $enable built-in. This is all we need to avoid an infinite \
recursion problem:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(BuiltinProcedure[x, y, {a, b}, myopt \[Rule] Automatic]\)], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("The variable $enablebuiltin is temporarily "\[InvisibleSpace]True
          \[InvisibleSpace]". \n The following function calls \
BuiltinProcedure without any condition on the arguments: it is the built-in \
one."\),
        SequenceForm[ 
        "The variable $enablebuiltin is temporarily ", True, 
          ". \n The following function calls BuiltinProcedure without any \
condition on the arguments: it is the built-in one."],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm
    \`myFun(BuiltinProcedure(x, y, {a, b}, myopt \[Rule] Automatic))\)], 
  "Output"]
}, Open  ]],

Cell["\<\
Incidentally, should we ever need to define a substitute for a \
built-in procudure in a recursive way, we would have to enclose our code \
inside a Block that sets the enironmental variable $enablebuiltin=False. We \
should not forget to provide a test to avoid infinite recursion, of course. \
We can also recode the builtin procedure in such a way that its arguments \
could call the redefined function itself. For example, the solution adopted \
by the package Summa.m, for allowing interpretation of nested Sums is the \
following:\
\>", "Text"],

Cell[TextData[{
  "Unprotect[Sum];\nClear[Sum];\nSum[args__]/;$EnableSum\[Equal]False:=\n\t\
Block[\n\t\t{",
  StyleBox["$EnableSum=True",
    FontWeight->"Bold"],
  "},\n\t\tSumma[Sequence@@Block[\n\t\t\t{",
  StyleBox["$EnableSum=False",
    FontWeight->"Bold"],
  "},\n\t\t\ttmp[args]\n\t\t\t]\n\t\t]\n    ]\nProtect[Sum];"
}], "Program",
  CellFrame->True,
  Background->GrayLevel[0.833326]]
}, Closed]],

Cell[CellGroupData[{

Cell["Applying functions to redefined built-in objects", "Section"],

Cell[TextData[{
  "The aforementioned method can be adopted to attach new definitions to any \
built-in object ",
  StyleBox["Obj",
    FontWeight->"Bold"],
  ". The simplest case is that in which a built-in object is transformed into \
a user defined wrapper ",
  StyleBox["userObj",
    FontWeight->"Bold"],
  " that can be used to ferry unevaluated arguments to other user defined \
functions ",
  StyleBox["userFuncts",
    FontWeight->"Bold"],
  ". This approach will avoid any premature evaluation allowing the user code \
to  preprocess the arguments to be passed to the original built-in Obj, \
post-process the result or to produce a final result on its own. It is thus \
possible to either adapt existing code (by adding options, or further error \
checking, for example) or to write brand new code from scratch to fully \
replace the built-in procedure."
}], "Text"],

Cell["\<\

A redefined object Obj can appear in the body of user defined functions. By \
setting the value of the environmental variable $enablebuiltin, the user can \
either call the built-in object Obj directly or his/her redefined code \
userObj. For example, if the environmental variable is normally set to False \
every call to the built-in object will be diverted to the user defined code, \
but a Block can be used to temporarly redefine $enablebuiltin prior to \
calling the built-in procedure\
\>", "Text"],

Cell[BoxData[
    \(userFunct[args_, opts___Rule] := 
      \[IndentingNewLine]Module[{}, 
        \[IndentingNewLine]\[IndentingNewLine] (*
          code\ that\ calls\ the\ user\ redefined\ procedure\ *) 
          \[IndentingNewLine]newArgs = f[args]; newOpts = g[opts]; 
        \[IndentingNewLine]userResult = BuiltinProcedure[newArgs, newOpts]; 
        \[IndentingNewLine]\[IndentingNewLine]Block[{$enablebuiltin = True}, 
          \[IndentingNewLine] (*\ 
            code\ that\ calls\ the\ built - in\ procedure*) 
            \[IndentingNewLine]newArgs = h[args]; newOpts = k[opts]; 
          \[IndentingNewLine]userResult = BuiltinProcedure[newArgs, newOpts]; 
          \[IndentingNewLine]]\[IndentingNewLine]]\)], "Input"],

Cell[TextData[{
  "\nIt is also possible to include the newly redefined object ",
  StyleBox["Obj",
    FontWeight->"Bold"],
  " as an argument of ",
  StyleBox["userFunct",
    FontWeight->"Bold"],
  ". If the usual evaluation process takes place (that is, unless the \
function has some holding attribute) and the environmental variable \
$enablebuiltin is set to False the arguments get evaluated and the user \
defined fuction will only 'see' the result of the user defined code ",
  StyleBox["userObj",
    FontWeight->"Bold"],
  ". If the code is just a wrapper it can be used to extract information that \
the user has put into what it thought it was the built-in object Obj.\nThe \
following sample code shows how to implement this scheme with a function ",
  StyleBox["userFunct",
    FontWeight->"Bold"],
  " acting on the redefined built-in object ",
  StyleBox["BuiltinProcedure",
    FontWeight->"Bold"],
  " prevously overridden by the user defined code ",
  StyleBox["userCode",
    FontWeight->"Bold"],
  ". All we have to pay attention to is to encapsulate the code that needs \
(if case may be) to call BuiltinProcedure inside a Block enabling built-in \
evaluation."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(userFunct[BuiltinProcedure[args_, opts___Rule], functarg_] := 
      \[IndentingNewLine]Block[{$enablebuiltin = True}, 
        \[IndentingNewLine] (*\ code\ acting\ on\ userCode\ output*) 
          \[IndentingNewLine] (*\ 
          can\ call\ the\ original\ built - in\ procedure*) 
          \[IndentingNewLine]Print[
          "\<$enablebuiltin is temporarily \>", $enablebuiltin]; 
        \[IndentingNewLine]f[BuiltinProcedure[args, opts]]\[IndentingNewLine]]
        \)], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("The variable $enablebuiltin is temporarily "\[InvisibleSpace]True
          \[InvisibleSpace]". \n The following function calls \
BuiltinProcedure without any condition on the arguments: it is the built-in \
one."\),
        SequenceForm[ 
        "The variable $enablebuiltin is temporarily ", True, 
          ". \n The following function calls BuiltinProcedure without any \
condition on the arguments: it is the built-in one."],
        Editable->False], TraditionalForm]], "Print"]
}, Open  ]],

Cell["\<\
The message is printed as a side effect of the evaluation of \
BuiltinProcedure in the arguments of userFunct: This means that there will no \
longer be signs of BuiltInProcedure in the definition of userFunct, as we can \
readily see by asking for the definition of userFunct:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?userFunct\)\)], "Input"],

Cell["Global`userFunct", "Print",
  CellTags->"Info3306191520-9087400"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {
                  \(userFunct[myFun[BuiltinProcedure[args_, opts___Rule]], 
                      functarg_] := 
                    Block[{$enablebuiltin = True}, 
                      Print["$enablebuiltin is temporarily ", \
$enablebuiltin]; f[BuiltinProcedure[args, opts]]]\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "userFunct"],
      Editable->False]], "Print",
  CellTags->"Info3306191520-9087400"]
}, Open  ]],

Cell[TextData[{
  "The signature matches that of ",
  StyleBox["userFunct[BuiltinProcedure, functarg]",
    FontWeight->"Bold"],
  " because both calls will undergo the same evaluation process. Hence \
BuiltinProcedure and UserProcedure are interchaghable as arguments of \
userFunct (as long as the environment has $enablebuiltin set to False). In \
the real world the function f and BuiltinProcedure will produce a result that \
would avoit too deep a nesting of arguments.\n\nUnder certain circumstances, \
it is even advisable to use ",
  StyleBox["userCode",
    FontWeight->"Bold"],
  " directly as an argument for userFunct. This is the case for the Format \
function, for example, because it would otherwise require to unprotect and \
protect the built-in object. "
}], "Text"],

Cell[BoxData[
    \(userFunct[userCode[args_, opts___Rule], functarg_] := 
      \[IndentingNewLine]Block[{$enablebuiltin = True}, 
        \[IndentingNewLine] (*\ code\ acting\ on\ userCode\ output*) 
          \[IndentingNewLine] (*\ 
          can\ call\ the\ original\ built - in\ procedure*) 
          \[IndentingNewLine]Print[
          "\<$enablebuiltin is temporarily \>", $enablebuiltin]; 
        \[IndentingNewLine]f[BuiltinProcedure[args, opts]]\[IndentingNewLine]]
        \)], "Input"],

Cell[TextData[{
  "As an aside we note that if the functions userFuncts are defined in terms \
of user defined code userCode, they will ",
  StyleBox["not",
    FontSlant->"Italic"],
  " apply to the original built-in object when the environmental variable \
$enablebuiltin is set to False. This facility allows for the cohesistence of \
two worlds and can be used as a basis for switching on and off the override \
of built-in functions."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Enabling and disabling built-in evaluation", "Section"],

Cell[TextData[{
  "Suppose we let the user defined code ",
  StyleBox["userObj",
    FontWeight->"Bold"],
  " that is intended to replace a built-in object ",
  StyleBox["Obj",
    FontWeight->"Bold"],
  " (it could be a wrapper, for example, or a procedure that eventually calls \
the built-in object) accessible to the global environment. When we set the \
environmental variable $enablebuiltin to True, we can access either the \
built-in object 'as is' and the user defined code. In a way we have turned \
the built-in evaluation for our object on."
}], "Text"],

Cell["\<\
If we don't want the user to know about the environmental variable \
at all, we can write a couple of naive procedures to unprotect and protect \
the environment variable for her.\
\>", "Text"],

Cell[BoxData[
    \(\(\ EnableObjEvaluation[] := 
      \[IndentingNewLine]\(\((\[IndentingNewLine]Unprotect[$enablebuiltin]; 
            \[IndentingNewLine]$enablebuiltin = True; 
            \[IndentingNewLine]Protect[$enablebuiltin]; \[IndentingNewLine])
            \)\n\[IndentingNewLine]DisableObjEvaluation[] := 
        \[IndentingNewLine]\((\[IndentingNewLine]Unprotect[$enablebuiltin]; 
          \[IndentingNewLine]$enablebuiltin = False; 
          \[IndentingNewLine]Protect[$enablebuiltin]; \[IndentingNewLine])
          \)\)\)\)], "Input",
  CellFrame->True,
  InitializationCell->True,
  Background->GrayLevel[0.833326]],

Cell["\<\
Moreover, we can make this process local to a single expression, \
regardless of the value of the environment variable $EnableObjEvaluation. For \
example, to locally enable Obj evaluation, we can let a procedure substitute \
every occurence of userObj with Obj when the built\[IGrave]-in evaluation is \
turned on and replacing the surviving Obj with userObj prior to outputting \
the result in an environment where the built-in Obj evaluaton is disabled. We \
must give this procedure the attibute Holdfirst to avoid the evaluation of \
the expression passed\
\>", "Text"],

Cell[BoxData[{
    \(\(Attributes[EnableObjEvaluation] = {HoldFirst}; \)\), 
    \(\[IndentingNewLine]\), 
    \(EnableObjEvaluation[expr_] := 
      \[IndentingNewLine]\((\[IndentingNewLine]EnableObjEvaluation[]; 
        \[IndentingNewLine]First[{expr /. userObj \[Rule] Obj, 
              DisableObjEvaluation[]}] /. 
          Obj \[Rule] userObj\[IndentingNewLine])\)\[IndentingNewLine]\), 
    \(\[IndentingNewLine]\), 
    \(\(Attributes[DisableObjEvaluation] = {HoldFirst}; \)\), 
    \(\[IndentingNewLine]\), 
    \(DisableObjEvaluation[expr_] := 
      \[IndentingNewLine]\((\[IndentingNewLine]DisableObjEvaluation[]; 
        \[IndentingNewLine]First[{expr /. Obj \[Rule] userObj, 
              EnableObjEvaluation[]}] /. 
          userObj \[Rule] Obj\[IndentingNewLine])\)\)}], "Input",
  CellFrame->True,
  InitializationCell->True,
  Background->GrayLevel[0.833326]],

Cell["\<\
Here we use First to give back a result after the last element of \
the list has been evaluated.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Overriding built-in procedures operating on overridden built-in \
objects *\
\>", "Section"],

Cell["\<\
Version 3 of the package Summa.m will go a further step forward by \
overriding the built-in functions Simplify, FullSimplify, D, Integrate, \
Expand, ExpandAll and a few others in order to enable the processing of Summa \
objects resulting from the redefinition of Sum. It will then be possible to \
differentiate a Sum by simply passing it to the usual D operator, for \
example.This is just a cosmetic enhancement that reduces the number of \
commands needed to manipulate this half new objects.To guarantee \
backcompability the older names will be retained.
The following predicate will be used to ascertain whether a 'specialized' \
procedure is to be invoked.\
\>", "Text"],

Cell[BoxData[
    \(SummaFreeQ[expr_] := FreeQ[expr, Summa, \[Infinity]]\)], "Input"],

Cell["\<\
The predicate needs to be generalized in order to let it accept a \
sequence of arguments. This is somewhat needed to automate the generation of \
all the necessary code.\
\>", "Text"],

Cell[BoxData[
    \(SummaFreeQ[args__] := \ FreeQ[{args}, Summa, \[Infinity]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(args = Sequence[pippo, pluto, paperino\  - Summa]; \n
    SummaFreeQ[args]\), 
    \(SummaFreeQ[pippo, pluto, paperino]\)}], "Input"],

Cell[BoxData[
    \(TraditionalForm\`False\)], "Output"],

Cell[BoxData[
    \(TraditionalForm\`True\)], "Output"]
}, Open  ]],

Cell["\<\
This is the enviromental variable used to turn on and off the \
overriding.\
\>", "Text"],

Cell[BoxData[
    \(\($enableSumSimplify = True; \)\)], "Input"],

Cell["\<\
After unprotecting a built-in procedure we can redefine it in such \
a way that it will be able to distinguish between arguments that contain \
Summa objects and arguments that do not. We will apply the user defined \
procedure only in the first case leaving the ordinary built-in procedure to \
the rest. FilterOption can be used to give the correct options to each \
procedure.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<Utilities`FilterOptions`\>"]\)], "Input"],

Cell[BoxData[{
    \(Unprotect[Simplify]; \n
    Simplify[args__, opts___Rule] /; $enableSumSimplify \[Equal] True := 
      \[IndentingNewLine]Block[{$enableSumSimplify = False}, 
        \[IndentingNewLine]If[SummaFreeQ[args], 
          \[IndentingNewLine]Simplify[args, FilterOptions[Simplify, opts]], 
          \[IndentingNewLine]SumSimplify[args, opts]\[IndentingNewLine]]
          \[IndentingNewLine]]\), 
    \(\(Protect[Simplify]; \)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[1 + Summa[x^k, {k, 1, \[Infinity]}]]\)], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"SumSimplify", "(", 
        RowBox[{
          RowBox[{"Summa", "(", 
            RowBox[{\(x\^k\), ",", 
              RowBox[{"{", 
                RowBox[{"k", ",", "1", ",", 
                  InterpretationBox["\[Infinity]",
                    DirectedInfinity[ 1]]}], "}"}]}], ")"}], "+", "1"}], 
        ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Further work is neeeded in order to let the options of the user \
defined procedure appeare between those of the built-in procedure. But that \
is beyond the scope of this notebook and will be covered in a separate \
notebook named \"Overriding.nb\", to be released later this century, when the \
authour will find the time to polish the override.m package.\
\>", "Text"],

Cell[BoxData[
    \(EnableSumProcedure[proc_] := 
      ToExpression[
        StringJoin[{"\<$EnableSum\>", ToString[proc], "\<=True\>"}]]; \n
    DisableSumProcedure[proc_] := 
      ToExpression[
        StringJoin[{"\<$EnableSum\>", ToString[proc], "\<=False\>"}]]; \)], 
  "Input"],

Cell[BoxData[
    \(\(EnableSumProcedure[Simplify]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \($EnableSumSimplify\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`True\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(DisableSumProcedure[Simplify]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \($EnableSumSimplify\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`False\)], "Output"]
}, Open  ]]
}, Closed]]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1152}, {0, 900}},
WindowToolbars->{},
WindowSize->{694, 529},
WindowMargins->{{221, Automatic}, {Automatic, 152}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
StyleDefinitions -> "Classroom.nb"]